#version 150 core

in vec2 textureCoord;

uniform sampler2D albedoBuffer;
uniform sampler2D normalBuffer;
uniform sampler2D lightBuffer;
uniform sampler2D depthBuffer;
uniform sampler2D ssaoBuffer;

uniform mat4 projectionMatrix;

uniform int   drawMode;
uniform float gammaCorrection;
uniform float exposure;
uniform bool  drawLights;
uniform bool  drawSSAO;
uniform vec3  ambientIntensity;

uniform bool homogeneousReversedDepth;

out vec4 color;

vec3 screenGamma = vec3( 1.0f / gammaCorrection );

float linearizeDepth( in float z ) {
  float A     = projectionMatrix[ 2 ].z;
  float B     = projectionMatrix[ 3 ].z;
  float zFar  = -B / ( 1.0 - A );
  float zNear = B / ( 1.0 + A );

  if ( homogeneousReversedDepth ) {
    return zNear / ( zNear - z * ( zNear - zFar ) );
  }
  return 2.0 * zNear / ( zFar + zNear - z * ( zFar - zNear ) );
}

vec3 getLighting() {
  vec3  light            = textureLod( lightBuffer, textureCoord, 0 ).rgb;
  float ssao             = drawSSAO ? textureLod( ssaoBuffer, textureCoord, 0 ).x : 1.0f;
  vec3  selfIllumination = vec3( textureLod( normalBuffer, textureCoord, 0 ).w );
  return max( light + ambientIntensity * ssao, selfIllumination );
}

// Exposure tone mapping
vec3 getHDR( in vec3 colorValue ) {
  vec3 mapped = vec3( 1.0f ) - exp( -colorValue * exposure );
  return pow( mapped, screenGamma );
}

void main() {
  if ( drawMode == 1 ) {
    vec3 colorSample = textureLod( albedoBuffer, textureCoord, 0 ).rgb;
    if ( drawLights ) {
      colorSample *= getLighting();
    }
    color = vec4( getHDR( colorSample ), 1.0f );
  } else if ( drawMode == 2 ) {
    color = vec4( textureLod( normalBuffer, textureCoord, 0 ).xyz, 1.0f );
  } else if ( drawMode == 3 ) {
    color = vec4( getHDR( getLighting() ), 1.0f );
  } else if ( drawMode == 4 ) {
    vec3 ssaoValue = vec3( textureLod( ssaoBuffer, textureCoord, 0 ).x );
    color          = vec4( ssaoValue, 1.0f );
  } else if ( drawMode == 5 ) {
    color = vec4( vec3( textureLod( depthBuffer, textureCoord, 0 ).x ), 1.0f );
  } else if ( drawMode == 6 ) {
    color = vec4( getHDR( textureLod( albedoBuffer, textureCoord, 0 ).rgb ), 1.0f );
  }
}
