#version 150

in VertexData {
  flat bool  isFont;
  flat float fontSmoothing;
  flat bool  hasOutline;
  flat float outlineWidth;
  flat bool  shouldPerformScissorTest;
  flat vec4  scissorInfo;
  vec2       textureCoord;
  flat vec2  textureCoordScale;
  flat vec2  textureCoordOffset;
  flat bool  isColor;
  flat vec4  color;
  flat vec4  secondColor;
  flat float opacity;
  flat float selfIllumination;
  mat3       tbnMatrix;
}
VertexIn;

layout( std140 ) uniform Parameters {
  mat4  viewProjectionMatrix;
  vec3  screenGamma;
  vec2  screenSize;
  float exposure;
  float alphaTestThreshold;
};

uniform sampler2D textureBuffer;

uniform bool isInterface;

out vec4 fragmentColor;

const float symbolWidth = 0.5f;

const vec3  lightColor = vec3( 1.0f, 1.0f, 1.0f );
const vec3  wsLightDir = normalize( vec3( -1.0f, 1.0f, 1.0f ) );
const float ambient    = 0.05;

// vec3 getNormal( in vec2 texCoord ) {
//   // переводим из tangent space в NDC [-1.0f, 1.0f]
//   vec3 ndcNormal = texture2D( textureNormal, texCoord ).xyz * 2.0f - 1.0f;
//   // переводим в мировое пространство
//   return normalize( VertexIn.tbnMatrix * ndcNormal );
// }

vec4 linearizeColor( in vec4 color ) {
  return vec4( pow( color.rgb, screenGamma ), color.a );
}

vec3 linearizeColor( in vec3 color ) {
  return pow( color, screenGamma );
}

vec4 gammaCorrectColor( in vec4 color ) {
  return vec4( pow( color.rgb, 1.0f / screenGamma ), color.a );
}

vec3 gammaCorrectColor( in vec3 color ) {
  return pow( color, 1.0f / screenGamma );
}

void main() {
  if ( VertexIn.shouldPerformScissorTest &&
    ( gl_FragCoord.x < VertexIn.scissorInfo.x || gl_FragCoord.x > VertexIn.scissorInfo.z || gl_FragCoord.y < VertexIn.scissorInfo.y ||
      gl_FragCoord.y > VertexIn.scissorInfo.w ) ) {
    discard;
  }

  vec2 texCoord = abs( fract( VertexIn.textureCoord ) ) * VertexIn.textureCoordScale + VertexIn.textureCoordOffset;
  // линеаризуем цвет для альбедо-текстур
  vec4 colorLinear;
  if ( VertexIn.isColor && VertexIn.isFont ) {
    colorLinear        = VertexIn.color;
    float distance     = textureGrad( textureBuffer, texCoord, dFdx( VertexIn.textureCoord ), dFdy( VertexIn.textureCoord ) ).a;
    float smoothFactor = smoothstep( symbolWidth - VertexIn.fontSmoothing, symbolWidth + VertexIn.fontSmoothing, distance );
    if ( VertexIn.hasOutline ) {
      colorLinear  = mix( VertexIn.secondColor, colorLinear, smoothFactor );
      smoothFactor = min( colorLinear.a,
        smoothstep( VertexIn.outlineWidth - VertexIn.fontSmoothing, VertexIn.outlineWidth + VertexIn.fontSmoothing, distance ) );
    }
    colorLinear = vec4( colorLinear.rgb, smoothFactor );
  } else if ( VertexIn.isColor ) {
    colorLinear = VertexIn.color;
  } else {
    // текстуры хранятся в буфере в нелинеаризованном виде, поэтому перед работой с ними их нужно линеаризовать
    // TODO: это временное решение, позже линеаризация будет осуществляться при чтении текстуры с диска
    colorLinear = linearizeColor( textureGrad( textureBuffer, texCoord, dFdx( VertexIn.textureCoord ), dFdy( VertexIn.textureCoord ) ) );
  }

  float alpha = min( colorLinear.a, VertexIn.opacity );
  if ( alpha < alphaTestThreshold ) {
    discard;
  }
  //vec3 finalColor = colorLinear.rgb;
  //  if ( shouldDrawLights ) {
  //    float diffuse = max( dot( getNormal( VertexIn.textureCoord ), wsLightDir ), 0.0f );
  //    vec3  light   = max( ( diffuse + ambient ), VertexIn.selfIllumination ) * lightColor;
  //    finalColor    = light * colorLinear.rgb;
  //  }

  if ( isInterface ) {
    fragmentColor = gammaCorrectColor( colorLinear );
  } else {
    vec3 mapped   = vec3( 1.0f ) - exp( -colorLinear.rgb * exposure );
    fragmentColor = vec4( gammaCorrectColor( mapped ), alpha );
  }
}
