#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

layout( location = 0 ) in vec3 inVertex;
layout( location = 1 ) in vec3 inTextureCoord;
layout( location = 4 ) in vec3 inBoneId;
layout( location = 5 ) in vec3 inWeight;

// это может быть матрица проекции обычной камеры или камеры источника света, поэтому передаем не через UBO
uniform mat4 viewProjectionMatrix;
uniform mat4 modelMatrix;
uniform bool homogeneousReversedDepth;
uniform bool isSky;

const int    maxBones   = 64;
const int    maxWeights = 2;
uniform mat4 boneMatrices[ maxBones ];

out vec2 textureCoord;

void main() {
  vec4 resultPosition = vec4( 0.0f );

  for ( int i = 0; i < maxWeights; ++i ) {
    mat4 boneTransform = boneMatrices[ int( inBoneId[ i ] ) ];
    vec4 localPosition = boneTransform * vec4( inVertex, 1.0f );
    resultPosition += localPosition * inWeight[ i ];
  }

  vec4 position = viewProjectionMatrix * modelMatrix * resultPosition;
  // if (isSky && homogeneousReversedDepth) {
  //   position.z = 0;
  // } else if (isSky) {
  //   position.z = position.w;
  // }
  gl_Position  = position;
  textureCoord = inTextureCoord.xy;
}
